/* Copyright (c) 2006 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 */

/** @file
 * Definition of the USB configuration descriptor type
 */

/** @{
 * @ingroup nordic_usb
 */
#ifndef  __USB_DESC_H__
#define  __USB_DESC_H__

#include <stdint.h>
#include <hal_usb_desc.h>

#define USB_DESC_TEMPLATE

//------------------------------------------
// Vendor ID and Product ID definitions
//------------------------------------------
#define VER_USB					0x0110	// USB specification revision
#define VID						  0x1915	// Vendor ID
#define PID						  0x5792	// Product ID
#define REVISION				0x0001	// Device Release number

// Descriptor type
#define DSC_TYPE_DEVICE					0x01
#define DSC_TYPE_CONFIG					0x02
#define DSC_TYPE_STRING					0x03
#define DSC_TYPE_INTERFACE			0x04
#define DSC_TYPE_ENDPOINT				0x05
#define DSC_TYPE_IAD					  0x0B

// Configuration descriptor: bmAttributes
#define DSC_CNFG_ATR_BASE				    0x80	// D7: base attribute, always 1
#define DSC_CNFG_ATR_SELF_POWERED		0x40	// D6: bus-powered: 0, self-powered: 1, both: 1
#define DSC_CNFG_ATR_BUS_POWERED		0x00
#define DSC_CNFG_ATR_REMOTEWAKEUP		0x20	// D5: remote-wakeup disabled: 0, enabled: 1

// Configuration descriptor: bMaxPower
#define DSC_CNFG_MAXPOWER( x )			(((x) + 1) / 2)		// 1 unit = 2mA

// Endpoint transfer type
#define DSC_EP_CONTROL				0x00
#define DSC_EP_ISOC						0x01
#define DSC_EP_BULK						0x02
#define DSC_EP_INTERRUPT			0x03

// Endopoint address
#define EP1_OUT							0x01
#define EP2_OUT							0x02
#define EP3_OUT							0x03
#define EP1_IN							0x81
#define EP2_IN							0x82
#define EP3_IN							0x83

// Descriptor type (Class specific)
#define DSC_TYPE_CS_INTERFACE			0x24

// Descriptor type (Class specific) - CDC
#define DSC_SUBTYPE_CS_CDC_HEADER_FUNC	0x00
#define DSC_SUBTYPE_CS_CDC_CALL_MAN		  0x01
#define DSC_SUBTYPE_CS_CDC_ABST_CNTRL	  0x02
#define DSC_SUBTYPE_CS_CDC_UNION_FUNC	  0x06

// Descriptor type (Class specific) - HID
#define DSC_SUBTYPE_CS_HID_CLASS		  0x21
#define DSC_SUBTYPE_CS_HID_REPORT		  0x22
#define DSC_SUBTYPE_CS_HID_PHYSICAL		0x23

#define USB_STRING_DESC_COUNT 3

//---------------------------------------------
// Header Functional Descriptor
//---------------------------------------------
typedef struct {
    uint8_t bLength;
    uint8_t bDescriptorType;
    uint8_t bDescriptorSubtype;
    uint16_t bcdCDC;
} Theader_func_descriptor;

//---------------------------------------------
// Call Management Functional Descriptor
//---------------------------------------------
typedef struct {
    uint8_t bLength;
    uint8_t bDescriptorType;
    uint8_t bDescriptorSubtype;
    uint8_t bmCapabilities;
    uint8_t bDataInterface;
} Tcall_man_func_descriptor;

//---------------------------------------------
// Abstract Control Management Functional Descriptor
//---------------------------------------------
typedef struct {
    uint8_t bLength;
    uint8_t bDescriptorType;
    uint8_t bDescriptorSubtype;
    uint8_t bmCapabilities;
} Tabst_control_mana_descriptor;

//---------------------------------------------
// Union Functional Descriptor
//---------------------------------------------
typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bDescriptorSubtype;
	uint8_t bMasterInterface;
	uint8_t bSlaveInterface0;
} Tunion_func_descriptor;

typedef struct {
	hal_usb_conf_desc_t				conf;
		hal_usb_if_desc_t				m_interface_desc_CDC_comm;
			Theader_func_descriptor			m_header_func_desc;
			Tcall_man_func_descriptor		m_call_man_desc;
			Tabst_control_mana_descriptor	m_abst_control_desc;
			Tunion_func_descriptor			m_union_func_desc;
			hal_usb_ep_desc_t			m_endpoint_desc_CDC_INT_IN;
		hal_usb_if_desc_t				m_interface_desc_CDC_data;
			hal_usb_ep_desc_t			m_endpoint_desc_CDC_BULK_IN;
			hal_usb_ep_desc_t			m_endpoint_desc_CDC_BULK_OUT;
} usb_conf_desc_templ_t;

typedef struct {
     volatile uint8_t* idx[USB_STRING_DESC_COUNT];
} usb_string_desc_templ_t;

extern code usb_string_desc_templ_t g_usb_string_desc;
extern code usb_conf_desc_templ_t g_usb_conf_desc;
extern code hal_usb_dev_desc_t g_usb_dev_desc;

typedef struct {
     hal_usb_dev_desc_t* dev;
     usb_conf_desc_templ_t* conf;
     usb_string_desc_templ_t* string;
     uint8_t string_zero[4];
} usb_descs_templ_t;

#endif  /* _USB_DESC_H_ */

/** @} */
