/* Copyright (c) 2007 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of 
 * Nordic Semiconductor. The use, copying, transfer or disclosure 
 * of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 */

/** @file
 * USB device descriptor and USB HID configuration descriptior.
 */

/** @{
 * @ingroup nordic_usb
 */


#include "usb_desc.h"
#include "nordic_common.h"

#define MAX_PACKET_SIZE_EP0  64

#define EP1_OUT_PACKET_SIZE			0x0040
#define EP2_OUT_PACKET_SIZE			0x0040
#define EP3_OUT_PACKET_SIZE			0x0040
#define EP1_IN_PACKET_SIZE			0x0040
#define EP2_IN_PACKET_SIZE			0x0040
#define EP3_IN_PACKET_SIZE			0x0040

enum {									// interface number
	DSC_INTERFACE_CDC_comm,
	DSC_INTERFACE_CDC_data,

	DSC_NUM_INTERFACE					// Number of Interfaces (for config desc)
};
//-----------------------------------------------------------------------------
// Descriptor Declarations
//-----------------------------------------------------------------------------

//
// Device descriptor
//
code const hal_usb_dev_desc_t g_usb_dev_desc =
{
	sizeof(hal_usb_dev_desc_t),				// bLength
	DSC_TYPE_DEVICE,						// bDescriptorType
	SWAP( VER_USB ),							// bcdUSB
	0x02,									// bDeviceClass (Communication Class)
	0x00,									// bDeviceSubClass
	0x00,									// bDeviceProtocol
	MAX_PACKET_SIZE_EP0,						// bMaxPacketSize0
	SWAP( VID ),								// idVendor
	SWAP( PID ),								// idProduct
	SWAP( REVISION ),							// bcdDevice
	0x01,									// iManufacturer
	0x02,									// iProduct
	0x04,									// iSerialNumber
	0x01,									// bNumConfigurations
}; //end of DeviceDesc
// From "USB Device Class Definition for Human Interface Devices (HID)".
// Section 7.1:
// "When a Get_Descriptor(Configuration) request is issued,
// it returns the Configuration descriptor, all Interface descriptors,
// all Endpoint descriptors, and the HID descriptor for each interface."

  // KOK
code const usb_conf_desc_templ_t g_usb_conf_desc =
{
  {										// Configuration descriptor
  	sizeof(hal_usb_conf_desc_t),		// bLength
  	DSC_TYPE_CONFIG,						// bDescriptorType
  	SWAP( sizeof(usb_conf_desc_templ_t) ),	// bTotalLength
  	DSC_NUM_INTERFACE,						// bNumInterfaces
  	0x01,									// bConfigurationValue
  	0x00,									// iConfiguration
  	  DSC_CNFG_ATR_BASE						// bmAttributes
  	| DSC_CNFG_ATR_BUS_POWERED,
  	DSC_CNFG_MAXPOWER( 100 ),				// bMaxPower (mA)
  },

  {										// Interface 0 - CDC Communication Class
  	sizeof(hal_usb_if_desc_t),			// bLength
  	DSC_TYPE_INTERFACE,						// bDescriptorType
  	DSC_INTERFACE_CDC_comm,					// bInterfaceNumber
  	0x00,									// bAlternateSetting
  	0x01,									// bNumEndpoints
  	0x02,									// bInterfaceClass (Communication Class)
  	0x02,									// bInterfaceSubClass (Abstract Control Model)
  	0x01,									// bInterfaceProcotol (V.25ter, Common AT commands)
  	0x00,									// iInterface
    },
  {										// Header Functional Descriptor
  	sizeof(Theader_func_descriptor),		// bLength
  	DSC_TYPE_CS_INTERFACE,					// bDescriptorType (CS_INTERFACE)
  	DSC_SUBTYPE_CS_CDC_HEADER_FUNC,			// bDescriptorSubtype (Header Functional)
  	SWAP(0x0110),								// bcdCDC (CDC spec release number, 1.1)
  },
  {										// Call Management Functional Descriptor
  	sizeof(Tcall_man_func_descriptor),		// bLength
  	DSC_TYPE_CS_INTERFACE,					// bDescriptorType (CS_INTERFACE)
  	DSC_SUBTYPE_CS_CDC_CALL_MAN,			// bDescriptorSubtype (Call Management)
  	0x01,									// bmCapabilities (only over Communication Class IF / handles itself)
  	DSC_INTERFACE_CDC_data,					// bDataInterface (Interface number of Data Class interface)
  },
  {										// Abstract Control Management Functional Descriptor
  	sizeof(Tabst_control_mana_descriptor),	// bLength
  	DSC_TYPE_CS_INTERFACE,					// bDescriptorType (CS_INTERFACE)
  	DSC_SUBTYPE_CS_CDC_ABST_CNTRL,			// bDescriptorSubtype (Abstract Control Management)
  	0x06,									// bmCapabilities (Supports Send_Break, Set_Line_Coding, Set_Control_Line_State,
  											//					Get_Line_Coding, and the notification Serial_State)
  },
  {										// Union Functional Descriptor
  	sizeof(Tunion_func_descriptor),			// bLength
  	DSC_TYPE_CS_INTERFACE,					// bDescriptorType (CS_INTERFACE)
  	DSC_SUBTYPE_CS_CDC_UNION_FUNC,			// bDescriptorSubtype (Union Functional)
  	DSC_INTERFACE_CDC_comm,					// bMasterInterface (Interface number master interface in the union)
  	DSC_INTERFACE_CDC_data,					// bSlaveInterface0 (Interface number slave interface in the union)
  },
  {										// Endpoint1
  	sizeof(hal_usb_ep_desc_t),			// bLength
  	DSC_TYPE_ENDPOINT,						// bDescriptorType
  	EP1_IN,									// bEndpointAddress
  	DSC_EP_INTERRUPT,						// bmAttributes
  	SWAP( EP1_IN_PACKET_SIZE ),				// MaxPacketSize
  	4										// bInterval
  },

  {										// Interface 1 - CDC Data Interface Class
  	sizeof(hal_usb_if_desc_t),			// bLength
  	DSC_TYPE_INTERFACE, 					// bDescriptorType
  	DSC_INTERFACE_CDC_data,					// bInterfaceNumber
  	0x00,									// bAlternateSetting
  	0x02,									// bNumEndpoints
  	0x0A,									// bInterfaceClass (Data Interface Class)
  	0x00,									// bInterfaceSubClass
  	0x00,									// bInterfaceProcotol (No class specific protocol required)
  	0x00									// iInterface
  },
  {										// Endpoint IN 2 descriptor
  	sizeof(hal_usb_ep_desc_t),			// bLength
  	DSC_TYPE_ENDPOINT,						// bDescriptorType
  	EP2_IN,									// bEndpointAddress
  	DSC_EP_BULK,							// bmAttributes
  	SWAP( EP2_IN_PACKET_SIZE ),				// MaxPacketSize
  	1										// bInterval
  },
  {										// Endpoint OUT 2 descriptor
  	sizeof(hal_usb_ep_desc_t),			// bLength
  	DSC_TYPE_ENDPOINT,						// bDescriptorType
  	EP2_OUT,								// bEndpointAddress
  	DSC_EP_BULK,							// bmAttributes
  	SWAP( EP2_OUT_PACKET_SIZE ),				// MaxPacketSize
  	1										// bInterval
  },
  // END OF KOK
};

#define USB_STRING_IDX_1_DESC "Nordic Semiconductor"
#define USB_STRING_IDX_2_DESC "Nordic Semiconductor USB Comport"
#define USB_STRING_IDX_3_DESC "USB Comport"

code uint8_t g_usb_string_desc_1[] = {
    sizeof(USB_STRING_IDX_1_DESC) * 2, 
    0x03, 
    'N',00,
    'o',00,
    'r',00,
    'd',00,
    'i',00,
    'c',00,
    ' ',00,
    'S',00,
    'e',00,
    'm',00,
    'i',00,
    'c',00,
    'o',00,
    'n',00,
    'd',00,
    'u',00,
    'c',00,
    't',00,
    'o',00,
    'r', 00 
};

code uint8_t g_usb_string_desc_2[] = {
    sizeof(USB_STRING_IDX_2_DESC) * 2,
    0x03, 
    'N',00,
    'o',00,
    'r',00,
    'd',00,
    'i',00,
    'c',00,
    ' ',00,
    'S',00,
    'e',00,
    'm',00,
    'i',00,
    'c',00,
    'o',00,
    'n',00,
    'd',00,
    'u',00,
    'c',00,
    't',00,
    'o',00,
    'r',00, 
    ' ',00, 
    'U',00,
    'S',00,
    'B',00,
    ' ',00,
    'C',00,
    'o',00,
    'm',00,
    'p',00,
    'o',00, 
    'r',00,
    't',00,
};

code uint8_t g_usb_string_desc_3[] = {
    sizeof(USB_STRING_IDX_3_DESC) * 2,
    0x03, 
    'U',00,
    'S',00,
    'B',00,
    ' ',00,
    'C',00,
    'o',00,
    'm',00,
    'p',00,
    'o',00,
    'r',00,
    't',00,
};

code usb_string_desc_templ_t g_usb_string_desc = {
    g_usb_string_desc_1,
    g_usb_string_desc_2,
    g_usb_string_desc_3,
};

/** @} */
